//
// Source File Name: JavaSamp.java  %I%
//
//  Licensed Materials -- Property of IBM
//
//  (c) Copyright International Business Machines Corporation, 1999.
//      All Rights Reserved.
//
//  US Government Users Restricted Rights -
//  Use, duplication or disclosure restricted by
//  GSA ADP Schedule Contract with IBM Corp.
//
//  PURPOSE :
//    Sample code to perform the following operations
//       - Connect to a database.
//       - Create a table and insert data
//       - Retrieve the data
//       - Drop the table
//       - Disconnect from the database.

// For more information about this sample, refer to the README file.

// For more information on programming in Java, refer to the 
// "Programming in Java" section of the Application Development Guide.

// For more information on building and running Java programs for DB2,
// refer to the "Building Java Applets and Applications" section of the 
// Application Building Guide.

// For more information on the SQL language, refer to the SQL Reference.


import java.io.*;
import java.lang.*;
import java.sql.*;

class JavaSamp {

   static {
      try {
         // register the driver with DriverManager
         // The newInstance() call is needed for the sample to work with
         // JDK 1.1.1 on OS/2, where the Class.forName() method does not
         // run the static initializer. For other JDKs, the newInstance
         // call can be omitted.
         Class.forName("COM.ibm.db2.jdbc.app.DB2Driver").newInstance();
      } catch (Exception e) {
         e.printStackTrace();
      }
   }

   public static void main (String[] argv) {
      Connection con = null;
      int NUM_ENTRY = 17;
      try {
         // connect to the server of user's choice
         // see Tools.class on Tools.DBConnect method
         if (argv.length == 3)
         {  // connect using command line arguments
            con = Tools.DBConnect(argv[0], argv[1], argv[2]); 
         }
         else
         {  // prompt user for database name, user ID, password
            con = Tools.DBConnect(); 
         }
         con.setAutoCommit (false);

         // create and populate table, then list information and drop table
         String tableName = "JAVA_SAMPLE";
         createTableWithValues (con, tableName);
         listColumnInfo (con, tableName);
         deleteTable (con, tableName);

         // disconnect from the connection made
         System.out.println ("\n>Disconnecting...");
         con.commit();
         con.close();
      } catch (Exception e) { e.printStackTrace(); }
   }

   //
   // createTableWithValues
   //    - create a new table and populate with a couple of rows of data
   //

   public static void createTableWithValues (Connection con, String tableName) {
      try {
         // create the table with one column
         Statement stmt = con.createStatement();
         String createTable = "CREATE TABLE " + tableName + " (COL1 VARCHAR(50))";
         System.out.println ("Create table - " + createTable);
         stmt.executeUpdate (createTable);
         stmt.close();

         // populate table with values "ROW 1" and "ROW 2"
         String insertValues = "INSERT INTO " + tableName + " VALUES (?)";
         System.out.println ("Insert - " + insertValues);
         PreparedStatement pstmt = con.prepareStatement (insertValues);
         pstmt.setString (1, "Row 1");
         pstmt.executeUpdate();
         pstmt.setString (1, "Row 2");
         pstmt.executeUpdate();
         pstmt.close();

         con.commit();
      } catch (Exception e) { e.printStackTrace(); }
   }

   //
   // listColumnInfo
   //    - determines the number of columns in the result set then lists
   //      name, type, precision, scale and nullability of each column
   //    - also lists values stored in first column
   //

   public static void listColumnInfo (Connection con, String tableName) {
      try {
         // select everything from the created table
         Statement stmt = con.createStatement();
         String selectRows = "SELECT * FROM " + tableName;
         ResultSet rs = stmt.executeQuery (selectRows);
         ResultSetMetaData rsmd = rs.getMetaData();
         System.out.println ("Select - " + selectRows);

         // determine the number of columns and list info for every column
         // in this case there is only one column
         int colNumber = rsmd.getColumnCount();
         System.out.println ("Number of columns - " + colNumber);
         String colNullable = new String("");

         // for every column the name, type, precision, scale,
         // nullability are listed
         for (int cnt = 1; cnt <= colNumber; cnt = cnt + 1) {
            String colName = rsmd.getColumnName (cnt);
            int colType = rsmd.getColumnType (cnt);
            int colPrecision = rsmd.getPrecision (cnt);
            int colScale = rsmd.getScale (cnt);
            if (rsmd.isNullable (cnt) == ResultSetMetaData.columnNullable) {
               colNullable = "TRUE";
            } else { colNullable = "FALSE"; }

            System.out.println ("Column name - " + colName);
            System.out.println ("Column type - " + colType);
            System.out.println ("Column precision - " + colPrecision);
            System.out.println ("Column scale - " + colScale);
            System.out.println ("Column nullable - " + colNullable);
         }
         // lists values for first column
         String colValue = new String("");
         while (rs.next()) {
            colValue = rs.getString (1);
            System.out.println ("Column value - " + colValue);
         }
         rs.close();
         stmt.close();
      } catch (Exception e) { e.printStackTrace(); }
   }

   //
   // deleteTable
   //    - drops the table from the server
   //

   public static void deleteTable (Connection con, String tableName) {
      try {
         String dropTable = "DROP TABLE " + tableName;
         Statement stmt = con.createStatement();
         stmt.executeUpdate (dropTable);
         System.out.println ("Delete table - " + dropTable);
         stmt.close();
         con.commit();
      } catch (Exception e) { e.printStackTrace(); }
   }
}
